@@ -1,5 +1,5 @@
 var EventSource = require('..')
-var es = new EventSource('http://localhost:8080/sse');
+var es = new EventSource('http://localhost:8080/sse')
 es.addEventListener('server-time', function (e) {
-  console.log(e.data);
-});
+  console.log(e.data)
+})
